/***************************************************************************
 * MoioChart Library - graphs and charts made easy                         *
 * Copyright (C) 2005 by Silvio Moioli                                     *
 * silvio@moioli.net, http://www.moioli.net                                *  
 *                                                                         *
 * This program is free software; you can redistribute it and/or           *
 * modify it under the terms of the GNU General Public License             *
 * as published by the Free Software Foundation; either version 2          *
 * of the License, or (at your option) any later version.                  *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program; if not, write to the Free Software             *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              *
 * MA  02110-1301, USA.                                                    *
 ***************************************************************************/

package net.moioli.chart.swing;

import swingwt.awt.Dimension;
import swingwt.awt.Graphics;
import swingwt.awt.Graphics2D;
import swingwt.awt.Rectangle;
import swingwt.awt.RenderingHints;
import java.util.Collection;
import java.util.Iterator;

import swingwtx.swing.JPanel;

import net.moioli.chart.ChartI;
import net.moioli.chart.DefaultChart;
import net.moioli.chart.IntRectangle;

/**
 * Draws a DefaultChart on a JPanel (Swing) object. <br />
 * It also Implements (Compatible)Collection, so that you can add(), addAll(),
 * retainAll(), clear(), etc. and it will automatically care about rescaling and
 * redrawing. Just drop this in your favourite application!
 * 
 * Patterns note: this class is a Decorator of the class DefaultChart
 * 
 * @author Silvio Moioli
 * @version 2005/10/5
 */
public class JGraph extends JPanel implements CompatibleCollection {
    /** The DefaultChart to be drawn */
    private DefaultChart g = null;

    /**
     * Default pixelAccuracy. It is set to the maximum value since if a computer
     * is running Swing, then it probably can handle a couple of more
     * operations.
     */
    public static final int ACCURACY = 1;

    /**
     * Constructs an empty JGraph.
     */
    public JGraph() {
        setGraph(new DefaultChart());
        getGraph().setPixelAccuracy(ACCURACY);
        setPreferredSize(new Dimension(300, 300));
    }

    /**
     * Constructs a multiple-object JGraph.
     * 
     * @param g
     *            the DefaultChart to be drawn
     */
    public JGraph(DefaultChart g) {
        setGraph(g);
        getGraph().setPixelAccuracy(ACCURACY);
        setPreferredSize(new Dimension(300, 300));
    }

    /**
     * Invoked by Swing to draw components.
     * 
     * @param jg
     *            the <code>Graphics</code> context in which to paint
     */
    public void paint(Graphics jg) {
        super.paint(jg);
        Rectangle r = getBounds();
        if ((r.getHeight() > 30) && (r.getWidth() > 30)) {
            drawMe(jg, r);
        }
    }

    /**
     * Clones this JGraph.
     * 
     * @return a clone of this instance.
     */
    public Object clone() {
        return new JGraph(this.g);
    }

    /**
     * Returns the DefaultChart drawn by this JGraph.
     * 
     * @return the DefaultChart drawn by this JGraph
     */
    public ChartI getGraph() {
        return this.g;
    }

    /**
     * Changes the drawn DefaultChart object.
     * 
     * @param g
     *            the new DefaultChart
     */
    public void setGraph(DefaultChart g) {
        this.g = g;
        repaint();
    }

    /**
     * Draws this object.
     * 
     * @param jg
     *            the surface where this JGraph will be drawn
     * @param clip
     *            the onscreen rectangle on which the DefaultChart will be drawn
     */
    public void drawMe(Graphics jg, Rectangle clip) {
        // Imposta i settaggi per il rendering (se possibile)
        if (jg instanceof Graphics2D) {
            Graphics2D g2d = (Graphics2D) jg;
            // RenderingHints rh = new RenderingHints(g2d.getRenderingHints());
            RenderingHints rh = g2d.getRenderingHints();
            rh.put(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
            rh.put(RenderingHints.KEY_TEXT_ANTIALIASING,
                    RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
            g2d.setRenderingHints(rh);
        }
        IntRectangle r2 = new IntRectangle(clip.x, clip.y, clip.width,
                clip.height);
        this.g.drawMe(new SwingGraphicsWrapper(jg), r2);
    }

    // Standard DefaultChart methods
    /**
     * @return Returns the maximum x value drawn.
     */
    public double getMaxX() {
        return this.g.getMaxX();
    }

    /**
     * @return Returns the minimum x value drawn.
     */
    public double getMinX() {
        return this.g.getMinX();
    }

    /**
     * @return Returns the maximum y value drawn.
     */
    public double getMaxY() {
        return this.g.getMaxY();
    }

    /**
     * @return Returns the minimum y value drawn.
     */
    public double getMinY() {
        return this.g.getMinY();
    }

    /**
     * @return Returns the pixelAccuracy value. To draw mathematical functions,
     *         DefaultChart will calculate the value of the function multiple
     *         times. This parameter assigns the the number of pixels on the x
     *         axis after which the y value will be recalculated. 1 is the
     *         minimum possible value (maximum precision), 3 is recommended in
     *         most cases.
     */
    public int getPixelAccuracy() {
        return this.g.getPixelAccuracy();
    }

    /**
     * @param maxX
     *            The maximum x value to be drawn.
     */
    public void setMaxX(double maxX) {
        this.g.setMaxX(maxX);
        repaint();
    }

    /**
     * @param minX
     *            The minimum x value to be drawn.
     */
    public void setMinX(double minX) {
        this.g.setMinX(minX);
        repaint();
    }

    /**
     * @param maxY
     *            The maximum y value to be drawn.
     */
    public void setMaxY(double maxY) {
        this.g.setMaxY(maxY);
        repaint();
    }

    /**
     * @param minY
     *            The minimum y value to be drawn.
     */
    public void setMinY(double minY) {
        this.g.setMinY(minY);
        repaint();
    }

    /**
     * @param pixelAccuracy
     *            The pixelAccuracy to set. To draw mathematical functions,
     *            DefaultChart will calculate the value of the function multiple
     *            times. This parameter assigns the the number of pixels on the
     *            x axis after which the y value will be recalculated. 1 is the
     *            minimum possible value (maximum precision), 3 is recommended
     *            in most cases.
     */
    public void setPixelAccuracy(int pixelAccuracy) {
        this.g.setPixelAccuracy(pixelAccuracy);
        repaint();
    }

    // Standard CompatibleCollection methods
    /**
     * Ensures that this collection contains the specified element (optional
     * operation). Returns <tt>true</tt> if this collection changed as a
     * result of the call. (Returns <tt>false</tt> if this collection does not
     * permit duplicates and already contains the specified element.)
     * <p>
     * 
     * Collections that support this operation may place limitations on what
     * elements may be added to this collection. In particular, some collections
     * will refuse to add <tt>null</tt> elements, and others will impose
     * restrictions on the type of elements that may be added. Collection
     * classes should clearly specify in their documentation any restrictions on
     * what elements may be added.
     * <p>
     * 
     * If a collection refuses to add a particular element for any reason other
     * than that it already contains the element, it <i>must </i> throw an
     * exception (rather than returning <tt>false</tt>). This preserves the
     * invariant that a collection always contains the specified element after
     * this call returns.
     * 
     * @param arg0
     *            element whose presence in this collection is to be ensured.
     * @return <tt>true</tt> if this collection changed as a result of the
     *         call
     * 
     * @throws UnsupportedOperationException
     *             <tt>add</tt> is not supported by this collection.
     * @throws ClassCastException
     *             class of the specified element prevents it from being added
     *             to this collection.
     * @throws NullPointerException
     *             if the specified element is null and this collection does not
     *             support null elements.
     * @throws IllegalArgumentException
     *             some aspect of this element prevents it from being added to
     *             this collection.
     */
    public boolean add(Object arg0) {
        boolean result = this.g.add(arg0);
        repaint();
        return result;
    }

    /**
     * Adds all of the elements in the specified collection to this collection
     * (optional operation). The behavior of this operation is undefined if the
     * specified collection is modified while the operation is in progress.
     * (This implies that the behavior of this call is undefined if the
     * specified collection is this collection, and this collection is
     * nonempty.)
     * 
     * @param arg0
     *            elements to be inserted into this collection.
     * @return <tt>true</tt> if this collection changed as a result of the
     *         call
     * 
     * @throws UnsupportedOperationException
     *             if this collection does not support the <tt>addAll</tt>
     *             method.
     * @throws ClassCastException
     *             if the class of an element of the specified collection
     *             prevents it from being added to this collection.
     * @throws NullPointerException
     *             if the specified collection contains one or more null
     *             elements and this collection does not support null elements,
     *             or if the specified collection is <tt>null</tt>.
     * @throws IllegalArgumentException
     *             some aspect of an element of the specified collection
     *             prevents it from being added to this collection.
     * @see #add(Object)
     */
    public boolean addAll(Collection arg0) {
        boolean result = this.g.addAll(arg0);
        repaint();
        return result;
    }

    /**
     * Removes all of the elements from this collection (optional operation).
     * This collection will be empty after this method returns unless it throws
     * an exception.
     * 
     * @throws UnsupportedOperationException
     *             if the <tt>clear</tt> method is not supported by this
     *             collection.
     */
    public void clear() {
        this.g.clear();
        repaint();
    }

    /**
     * Returns <tt>true</tt> if this collection contains the specified
     * element. More formally, returns <tt>true</tt> if and only if this
     * collection contains at least one element <tt>e</tt> such that
     * <tt>(o==null ? e==null : o.equals(e))</tt>.
     * 
     * @param arg0
     *            element whose presence in this collection is to be tested.
     * @return <tt>true</tt> if this collection contains the specified element
     * @throws ClassCastException
     *             if the type of the specified element is incompatible with
     *             this collection (optional).
     * @throws NullPointerException
     *             if the specified element is null and this collection does not
     *             support null elements (optional).
     */
    public boolean contains(Object arg0) {
        return this.g.contains(arg0);
    }

    /**
     * Returns <tt>true</tt> if this collection contains all of the elements
     * in the specified collection.
     * 
     * @param arg0
     *            collection to be checked for containment in this collection.
     * @return <tt>true</tt> if this collection contains all of the elements
     *         in the specified collection
     * @throws ClassCastException
     *             if the types of one or more elements in the specified
     *             collection are incompatible with this collection (optional).
     * @throws NullPointerException
     *             if the specified collection contains one or more null
     *             elements and this collection does not support null elements
     *             (optional).
     * @throws NullPointerException
     *             if the specified collection is <tt>null</tt>.
     * @see #contains(Object)
     */
    public boolean containsAll(Collection arg0) {
        return this.g.containsAll(arg0);
    }

    /**
     * Returns <tt>true</tt> if this collection contains no elements.
     * 
     * @return <tt>true</tt> if this collection contains no elements
     */
    public boolean isEmpty() {
        return this.g.isEmpty();
    }

    /**
     * Returns an iterator over the elements in this collection. There are no
     * guarantees concerning the order in which the elements are returned
     * (unless this collection is an instance of some class that provides a
     * guarantee).
     * 
     * @return an <tt>Iterator</tt> over the elements in this collection
     */
    public Iterator iterator() {
        return this.g.iterator();
    }

    /**
     * Removes a single instance of the specified element from this collection,
     * if it is present (optional operation). More formally, removes an element
     * <tt>e</tt> such that <tt>(o==null ?  e==null :
     * o.equals(e))</tt>, if
     * this collection contains one or more such elements. Returns true if this
     * collection contained the specified element (or equivalently, if this
     * collection changed as a result of the call).
     * 
     * @param arg0
     *            element to be removed from this collection, if present.
     * @return <tt>true</tt> if this collection changed as a result of the
     *         call
     * 
     * @throws ClassCastException
     *             if the type of the specified element is incompatible with
     *             this collection (optional).
     * @throws NullPointerException
     *             if the specified element is null and this collection does not
     *             support null elements (optional).
     * @throws UnsupportedOperationException
     *             remove is not supported by this collection.
     */
    public boolean remove(Object arg0) {
        boolean result = this.g.remove(arg0);
        repaint();
        return result;
    }

    /**
     * 
     * Removes all this collection's elements that are also contained in the
     * specified collection (optional operation). After this call returns, this
     * collection will contain no elements in common with the specified
     * collection.
     * 
     * @param arg0
     *            elements to be removed from this collection.
     * @return <tt>true</tt> if this collection changed as a result of the
     *         call
     * 
     * @throws UnsupportedOperationException
     *             if the <tt>removeAll</tt> method is not supported by this
     *             collection.
     * @throws ClassCastException
     *             if the types of one or more elements in this collection are
     *             incompatible with the specified collection (optional).
     * @throws NullPointerException
     *             if this collection contains one or more null elements and the
     *             specified collection does not support null elements
     *             (optional).
     * @throws NullPointerException
     *             if the specified collection is <tt>null</tt>.
     * @see #remove(Object)
     * @see #contains(Object)
     */
    public boolean removeAll(Collection arg0) {
        boolean result = this.g.removeAll(arg0);
        repaint();
        return result;
    }

    /**
     * Retains only the elements in this collection that are contained in the
     * specified collection (optional operation). In other words, removes from
     * this collection all of its elements that are not contained in the
     * specified collection.
     * 
     * @param arg0
     *            elements to be retained in this collection.
     * @return <tt>true</tt> if this collection changed as a result of the
     *         call
     * 
     * @throws UnsupportedOperationException
     *             if the <tt>retainAll</tt> method is not supported by this
     *             Collection.
     * @throws ClassCastException
     *             if the types of one or more elements in this collection are
     *             incompatible with the specified collection (optional).
     * @throws NullPointerException
     *             if this collection contains one or more null elements and the
     *             specified collection does not support null elements
     *             (optional).
     * @throws NullPointerException
     *             if the specified collection is <tt>null</tt>.
     * @see #remove(Object)
     * @see #contains(Object)
     */
    public boolean retainAll(Collection arg0) {
        boolean result = this.g.retainAll(arg0);
        repaint();
        return result;
    }

    /**
     * Returns an array containing all of the elements in this collection. If
     * the collection makes any guarantees as to what order its elements are
     * returned by its iterator, this method must return the elements in the
     * same order.
     * <p>
     * 
     * The returned array will be "safe" in that no references to it are
     * maintained by this collection. (In other words, this method must allocate
     * a new array even if this collection is backed by an array). The caller is
     * thus free to modify the returned array.
     * <p>
     * 
     * This method acts as bridge between array-based and collection-based APIs.
     * 
     * @return an array containing all of the elements in this collection
     */
    public Object[] toArray() {
        return this.g.toArray();
    }

    /**
     * Returns an array containing all of the elements in this collection; the
     * runtime type of the returned array is that of the specified array. If the
     * collection fits in the specified array, it is returned therein.
     * Otherwise, a new array is allocated with the runtime type of the
     * specified array and the size of this collection.
     * <p>
     * 
     * If this collection fits in the specified array with room to spare (i.e.,
     * the array has more elements than this collection), the element in the
     * array immediately following the end of the collection is set to
     * <tt>null</tt>. This is useful in determining the length of this
     * collection <i>only </i> if the caller knows that this collection does not
     * contain any <tt>null</tt> elements.)
     * <p>
     * 
     * If this collection makes any guarantees as to what order its elements are
     * returned by its iterator, this method must return the elements in the
     * same order.
     * <p>
     * 
     * Like the <tt>toArray</tt> method, this method acts as bridge between
     * array-based and collection-based APIs. Further, this method allows
     * precise control over the runtime type of the output array, and may, under
     * certain circumstances, be used to save allocation costs
     * <p>
     * 
     * Suppose <tt>l</tt> is a <tt>List</tt> known to contain only strings.
     * The following code can be used to dump the list into a newly allocated
     * array of <tt>String</tt>:
     * 
     * <pre>
     * String[] x = (String[]) v.toArray(new String[0]);
     * </pre>
     * 
     * <p>
     * 
     * Note that <tt>toArray(new Object[0])</tt> is identical in function to
     * <tt>toArray()</tt>.
     * 
     * @param arg0
     *            the array into which the elements of this collection are to be
     *            stored, if it is big enough; otherwise, a new array of the
     *            same runtime type is allocated for this purpose.
     * @return an array containing the elements of this collection
     * 
     * @throws ArrayStoreException
     *             the runtime type of the specified array is not a supertype of
     *             the runtime type of every element in this collection.
     * @throws NullPointerException
     *             if the specified array is <tt>null</tt>.
     */
    public Object[] toArray(Object[] arg0) {
        return this.g.toArray(arg0);
    }

    /**
     * Returns the number of elements in the Collection.
     * 
     * @return the number of elements
     */
    public int getCollectionSize() {
        return this.g.size();
    }
}